#!/bin/sh
# Bryant Hansen

ME="$(basename "$0")"
USAGE="$ME [ -v ]"

UPLOAD_LOG=/var/log/logfile_uploader.log

KNOWN_HOSTS_FILE=/home/root/.ssh/known_hosts
SERVER_HOST_KEYFILE=/home/root/.ssh/known_hosts_key.txt


DebugMessage() {
    local LOG_RECORD_HEADER="`date "+%Y%m%d_%H%M%S"` (${ME}):"
    echo "${LOG_RECORD_HEADER} $*" >&2
}


CONF_FILE="/etc/logfile_uploader.conf"
if [[ ! -f "$CONF_FILE" ]] ; then
        DebugMessage "ERROR: configuration file $CONF_FILE not found."
    if [[ -f "$PWD"/logfile_uploader.conf ]] ; then
        DebugMessage "installing configuration file from $PWD to $CONF_FILE"
        cp -a "$PWD"/logfile_uploader.conf /etc/
    elif [[ -f "$(dirname "$0")"/logfile_uploader.conf ]] ; then
        DebugMessage "installing configuration file from $(dirname "$0") to $CONF_FILE"
        cp -a "$(dirname "$0")"/logfile_uploader.conf /etc/
    else
        DebugMessage "ERROR: cannot locate configuration file logfile_uploader.conf"
        DebugMessage "Exiting abnormally."
        exit 2
    fi
fi
. "$CONF_FILE"


VERBOSE=""
if [[ "$1" == "-v" ]] ; then
    VERBOSE=1
    shift
fi


mac=$(get_mac_address.sh)
if [[ ! "$mac" ]] ; then
    DebugMessage "ERROR: failed to retrieve mac address of localhost."
    DebugMessage "  Exiting abnormally."
    exit 5
fi
USER=$mac
if [[ ! "${USER%%[0-9]*}" ]] ; then
        DebugMessage "$ME ERROR: MAC address does not begin with a letter.  Modifying registration name from $USER to m_${USER}"
        USER="m_${USER}"
fi

if ! add_crontab_entry.sh ; then
    echo "ERROR $? adding entry to crontab" >&2
fi

# verify the server host key in the known_hosts file and add it if it's not there
if grep "^${SERVER} ssh-rsa.*$" $KNOWN_HOSTS_FILE > /dev/null 2>&1 ; then
    DebugMessage "the server known_host key is already installed in the known_hosts file $KNOWN_HOSTS_FILE"
else
    DebugMessage "server known_host key does not exist in KNOWN_HOSTS_FILE $KNOWN_HOSTS_FILE"
    echo "cat $SERVER_HOST_KEYFILE >> $KNOWN_HOSTS_FILE"
    sleep 1 # this seems to be necessary when sending commands to stdout
    if ! grep "^${SERVER} ssh-rsa.*$" $KNOWN_HOSTS_FILE > /dev/null 2>&1 ; then
        DebugMessage "ERROR: failed to install server host key into known_hosts file $KNOWN_HOSTS_FILE"
        exit 6
    fi
fi

KEYFILE=/home/root/.ssh/${USER}.key
if [[ ! -f "$KEYFILE" ]] ; then
    DebugMessage "NOTICE: KEYFILE $KEYFILE not found.  Generating..."
    dropbearkey -t rsa -f $KEYFILE | grep "^ssh-rsa\ .*$" > ${KEYFILE}.pub
    chmod 600 ${KEYFILE}.pub
    if [[ ! -f "$KEYFILE" ]] ; then
        DebugMessage "ERROR: failed to generate KEYFILE $KEYFILE"
        DebugMessage "  Exiting abnormally."
        exit 7
    fi
    DebugMessage "Generated KEYFILE $KEYFILE"
fi

# verify that ssh is functioning
DebugMessage "checking for ssh connectability to ${SERVER}..."
if ! ssh -y $SSH_OPTS -i $KEYFILE $USER@$SERVER ls -al /home/$USER > /dev/null 2>&1 ; then
    DebugMessage "no access to account on server.  Attempting new account registration..."
    do_registration.sh
    DebugMessage "new account registration pending"
    exit 8
else
    [[ "$VERBOSE" ]] && DebugMessage "ssh connection succeeded"
fi

exit 0
