#!/bin/sh
#   Copyright (c) 2010 digitalSTROM.org, Zurich, Switzerland
#
#   Author: Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
#
#   This file is part of digitalSTROM Server.
#
#   netinfo.sh is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   netinfo.sh is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this file. If not, see <http://www.gnu.org/licenses/>.

#   Note: when called with "upgrade", "addon-upgrade" or "addon-install"
#   parameter, the "update" command is always executed first.

. /www/pages/cgi-bin/errlib.sh
. /www/pages/cgi-bin/util.sh

export PATH=/usr/local/bin:/usr/bin:/bin:/usr/local/sbin:/usr/sbin:/sbin
export SHELL=/bin/sh

LOCKFILE="/var/lock/systemupdate"
LOGDIR="/var/log/dss11-websetup"
mkdir -p $LOGDIR
UPDATE_CODE="$LOGDIR/last-update-code-dsmfw.txt"
UPDATE_MESSAGE="$LOGDIR/last-update-msg-dsmfw.txt"
TIMEOUT=10

CMD=perform-dsm-upgrade

NOLOCK=0

# avoid conflict in util.sh
dsmlog()
{
    echo "[`date`] $1"
}

signal_led_update() {
    if [[ ! -x /usr/bin/dss11-ledd-client ]]; then
        return 1
    fi

    if [[ "$1" == "start" ]]; then
        /usr/bin/dss11-ledd-client system_update_started
    elif [[ "$1" == "end" ]]; then
        /usr/bin/dss11-ledd-client system_update_completed
    fi
}

dsmlog "dsmfwupgrade.sh script starts -----"
if [[ "$1" == "nolock" ]]; then
    NOLOCK=1
    dsmlog "Detected \"nolock\" option, will not modify lock files."
fi

I=0
while [[ "`cat /etc/runit/ds485d/supervise/stat`" != "run"  && $I -lt $TIMEOUT ]]; do
    dsmlog "Wait for dS485d to become available ..."
    usleep 2000000
    let I+=1
done

if [[ ! -x /usr/bin/perform-dsm-upgrade ]]; then
    dsmlog "perform-dsm-upgrade script not found!"
    dsmlog "dsmfwupgrade.sh script ends -----"
    exit $ERR_SETUP
fi

if [[ "$NOLOCK" -eq "0" ]] && [[ -e "$LOCKFILE" ]]; then
    dsmlog "Lock file exists, update is in progress..."
    dsmlog "dsmfwupgrade.sh script ends -----"
    exit $ERR_DOWAIT
fi

# we are good to go
if [[ "$NOLOCK" -eq "0" ]]; then
    touch "$LOCKFILE"
    signal_led_update "start"
    set_update_code $UPDATE_STATUS_DSM_FIRMWARE
fi

set_update_message "Updating dSM firmware..."

# stop dss for the time of the dSM update
if [[  -x "/usr/sbin/sv" ]] && [[ -d "/etc/runit/dss" ]]; then
    dsmlog "Stopping dSS..."
    sv -w 10 force-stop /etc/runit/dss
    if [[ $? -ne 0 ]]; then
        dsmlog "sv command returned error, killing dSS process"
        killall -9 dss
    fi
fi

# stop dsa for the time of the dSM update
if [[  -f "/usr/sbin/sv" ]] && [[ -d "/etc/runit/dsa" ]]; then
    dsmlog "Stopping dSA..."
    sv -w 10 force-stop /etc/runit/dsa
    if [[ $? -ne 0 ]]; then
        dsmlog "sv command returned error, killing dSA process"
        killall -9 dsa
    fi
fi

# safeguard to make sure dSS is really down
if [[ -n "`pidof dss`" ]]; then
    dsmlog "dSS still running, killing with -9"
    killall -9 dss
    usleep 1000000
fi

# safeguard to make sure dSA is really down
if [[ -n "`pidof dsa`" ]]; then
    dsmlog "dSA still running, killing with -9"
    killall -9 dsa
    usleep 1000000
fi

dsmlog "Executing: \"$CMD\""
$CMD
RETCODE="$?"

if [[  -x "/usr/sbin/sv" ]] && [[ -d "/etc/runit/dss" ]]; then
    dsmlog "Starting dSS..."
    sv up /etc/runit/dss
    if [[ $? -ne 0 ]]; then
        dsmlog "sv command return error, could not start dSS!"
    fi
else
    dsmlog "Error: can not start dSS!"
fi

if [[  -x "/usr/sbin/sv" ]] && [[ -d "/etc/runit/dsa" ]]; then
    dsmlog "Starting dSA..."
    sv up /etc/runit/dsa
    if [[ $? -ne 0 ]]; then
        dsmlog "sv command return error, could not start dSA!"
    fi
else
    dsmlog "Error: can not start dSA!"
fi

if [[ $RETCODE -ge 128 ]]; then
    echo "$ERR_UPDATE:$RETCODE" > $UPDATE_CODE
    echo "Error when attempting to perform dSM firmware upgrade" > $UPDATE_MESSAGE
    dsmlog "Error when attempting to perform dSM firmware upgrade"
    if [[ "$NOLOCK" -eq 0 ]]; then
        rm -f "$LOCKFILE"
        signal_led_update "end"
    fi
    dsmlog "dsmfwupgrade.sh script ends -----"
    exit $ERR_UPDATE
fi

if [[ "$NOLOCK" -eq 0 ]]; then
    rm -f "$LOCKFILE"
    signal_led_update "end"
fi

set_update_message
dsmlog "dsmfwupgrade.sh script ends -----"


echo "$ERR_OK:$RETCODE" > $UPDATE_CODE
echo "OK" > $UPDATE_MESSAGE
exit $ERR_OK
